/* 
 * File: MainController.js
 * Container controller for the entire world
 */

/*jslint node: true, vars: true, bitwise: true */
/*global angular, document, ClassExample, Camera, CanvasMouseSupport */
/* find out more about jslint: http://www.jslint.com/help.html */

"use strict";

// Creates the "backend" logical support for appMyExample
var myModule = angular.module("appMyExample", ["CSS450Timer", "CSS450Slider", "CSS450Xform"]);

// registers the constructor for the controller
// NOTE: the constructor is only called _AFTER_ the </body> tag is encountered
//       this code does NOT run until the end of loading the HTML page
myModule.controller("MainCtrl", function ($scope) {
    // Initialize the graphics system
    gEngine.Core.initializeWebGL('GLCanvas');
    $scope.mCanvasMouse = new CanvasMouseSupport('GLCanvas');
    
    $scope.mMyWorld = new ClassExample();
    $scope.mFileTexture = $scope.mMyWorld.getFileTexture();
    
    var gl = gEngine.Core.getGL();
    $scope.kTextureWrap = [
        {label: "Repeat",   value: gl.REPEAT},
        {label: "Mirrored", value: gl.MIRRORED_REPEAT},
        {label: "Clamp",    value: gl.CLAMP_TO_EDGE}
    ];
    
    $scope.kMagFilter = [
        {label: "Linear",       value: gl.LINEAR},
        {label: "Nearest",      value: gl.NEAREST}
    ];
    
    $scope.kMinFilter = [
        {label: "Linear",     value: gl.LINEAR},
        {label: "Nearest",    value: gl.NEAREST},
        {label: "N-Mipmap-N", value: gl.NEAREST_MIPMAP_NEAREST},
        {label: "L-Mipmap-N", value: gl.LINEAR_MIPMAP_NEAREST},
        {label: "N-Mipmap-L", value: gl.NEAREST_MIPMAP_LINEAR},
        {label: "L-Mipmap-L", value: gl.LINEAR_MIPMAP_LINEAR}
    ];
    
    $scope.mView = new Camera(
         [40, 30],
         80,
         [0, 0, 800, 600]);
    $scope.mBlendColor = false;
    
    $scope.mTextureStage = "0";
    $scope.mSelectedXform = $scope.mMyWorld.currentObject().getTextureXform($scope.mTextureStage);
    $scope.mMyImagePath = null;
        
    $scope.mainTimerHandler = function () {
        // $scope.mMyWorld.update();
        gEngine.Core.clearCanvas([0.9, 0.9, 0.9, 1]);        // Clear the canvas
        $scope.mMyWorld.draw($scope.mView);
    };

    $scope.defineSquare = function (event) {
        switch (event.which) {
        case 1: // left
            $scope.mMyWorld.defineCenter(
                $scope.mView.mouseWCX($scope.mCanvasMouse.getPixelXPos(event)),
                $scope.mView.mouseWCY($scope.mCanvasMouse.getPixelYPos(event)));
            $scope.mForceRedraw = true;
            $scope.selectTexture();
            break;
        case 2: // middle
            var xf = $scope.mMyWorld.currentObject().getXform();
            xf.setPosition(
                    $scope.mView.mouseWCX($scope.mCanvasMouse.getPixelXPos(event)),
                    $scope.mView.mouseWCY($scope.mCanvasMouse.getPixelYPos(event))
                    );
            break;
        }
    };

    $scope.dragSquare = function (event) {
        // console.log("dragging");
        switch (event.which) {
        case 1: // left
            $scope.mMyWorld.defineWidth(
                $scope.mView.mouseWCX($scope.mCanvasMouse.getPixelXPos(event)),
                $scope.mView.mouseWCX($scope.mCanvasMouse.getPixelYPos(event)));
            $scope.mForceRedraw = true;
            $scope.setBlendColor();
            break;
         case 2: // middle
            var xf = $scope.mMyWorld.currentObject().getXform();
            xf.setPosition(
                    $scope.mView.mouseWCX($scope.mCanvasMouse.getPixelXPos(event)),
                    $scope.mView.mouseWCY($scope.mCanvasMouse.getPixelYPos(event))
                    );
            break;
        }
    };
    
    $scope.acceptFiles = function (event) {
        var input = event.target;
        var reader = new FileReader();
        reader.onload = function () {
            // hacky for now
            $scope.mMyImage = new Image();
            $scope.mMyImage.src = reader.result;
            $scope.mMyWorld.newFileTexture(reader.result, false);
            $scope.mTextureStage = $scope.mMyWorld.currentObject().getNumTextures()-1;
            $scope.selectTexture();
        };
        $scope.mMyImagePath = input.files[0];
        reader.readAsDataURL(input.files[0]);
    };
    
    $scope.setBlendColor = function () {
        $scope.mMyWorld.currentObject().setBlendColor($scope.mBlendColor);
    };
    
    $scope.selectTexture = function () {
        // $scope.mTextureStage = parseInt($scope.mTextureStage);
        if ($scope.mTextureStage >= $scope.mMyWorld.currentObject().getNumTextures())
            $scope.mTextureStage = $scope.mMyWorld.currentObject().getNumTextures()-1;
        var n = $scope.mTextureStage;
        $scope.mFileTexture = $scope.mMyWorld.currentObject().getFileTexture(n);
        $scope.mSelectedXform = $scope.mMyWorld.currentObject().getTextureXform(n);
    };

});